/******************************* sci.c *******************************/

// Functions for use with SCI to perform serial communications
 
#include <mc9s12a64.h>
#include "SCI.h"
#include "constants.h"


char PrintBuffer[PRINT_BUFFER_SIZE];   

// port can be either 0 or 1


// sets the serial port baud rate to anything you like
// taking into account the system clock frequency
void SCI_Init(char port, unsigned long int BaudRate)
{
   BaudRate = (CLOCK_SPEED / 16) / BaudRate;  // determine the frequency divisor
  
   if (port == 0) 
   {	       
      SCI0BDH = (unsigned char)(BaudRate >> 8);   // set high portion of divisor 
      SCI0BDL = BaudRate & 0xFF;                  // set low portion of divisor
      SCI0CR1 = 0;                                // various transmission settings
      SCI0CR2 = 0x0C;                             // enable transmit and receive
   }
   else if (port == 1) 
   {	       
      SCI1BDH = (unsigned char)(BaudRate >> 8);   // set high portion of divisor 
      SCI1BDL = BaudRate & 0xFF;                  // set low portion of divisor
      SCI1CR1 = 0;                                // various transmission settings
      SCI1CR2 = 0x0C;                             // enable transmit and receive
   }
}
   
   
// wait for data to fill SCI data register then return it
char SCI_InChar(char port)
{
   if (port == 0) 
   {       
      while (!(SCI0SR1 & RDRF));
      return SCI0DRL;
   }
   else if (port == 1) 
   {       
      while (!(SCI1SR1 & RDRF));
      return SCI1DRL;
   }
}
														  
        
// wait for SCI data register to be empty, then fill it with the input value to send it over SCI
void SCI_OutChar(char port, char data)
{
   if (port == 0) 
   {        
      while (!(SCI0SR1 & TDRE));
      SCI0DRL = data;
   }
   else if (port == 1) 
   {        
      while (!(SCI1SR1 & TDRE));
      SCI1DRL = data;
   }
}     



// Outputs a null terminated string over SCI
void SCI_OutString(char port, char *pVal)
{            
   while(*pVal)
   {
      SCI_OutChar(port, *pVal);
      pVal++;
   }	    
}


// Outputs a 16bit unsigned integer (0-65536)
void SCI_OutUInt(char port, unsigned short n)
{	 
   unsigned char i=0;
     
   do
   {
      PrintBuffer[i++] = n % 10;
      n /= 10;   
   } while (n);
   
   do
   { 
      SCI_OutChar(port, PrintBuffer[--i] + '0');
   } while (i);
}



// Outputs a 32bit unsigned integer
void SCI_OutULInt(char port, unsigned long n)
{	 
   unsigned char i=0;
     
   do
   {
      PrintBuffer[i++] = n % 10;
      n /= 10;   
   } while (n);
   
   do
   { 
      SCI_OutChar(port, PrintBuffer[--i] + '0');
   } while (i);
}


// Outputs a 16bit unsigned integer
// fit to a certain minimum width
void SCI_OutUIntF(char port, unsigned short n, unsigned char width)
{	 
   unsigned char i=0;
   
   // determine digits  
   do
   {
      PrintBuffer[i++] = n % 10;
      n /= 10;   
   } while (n);
   
   // pad with spaces
   while (width > i)
   {
      SCI_OutChar(port, ' ');
      width--;
   }   
   
   // output digits
   do
   { 
      SCI_OutChar(port, PrintBuffer[--i] + '0');
   } while (i);
}


// prints a character as hex
void SCI_OutHex(char port, unsigned char data)
{
   unsigned H, L;
   L = data & 0xF;
   H = data >> 4;
  
   if (H > 9)
      SCI_OutChar(port, H - 10 + 'A');
   else
      SCI_OutChar(port, H + '0');
    
   if (L > 9)
      SCI_OutChar(port, L - 10 + 'A');
   else
      SCI_OutChar(port, L + '0');
}
