/******************************* camera.h *******************************/

#ifndef HEADER_CAMERA
#define HEADER_CAMERA


#define CAMERA_INPUT_PORT PTAD
#define CAMERA_OUTPUT_PORT PTT

#define CAMERA_MASK_ON (0x04)
#define CAMERA_MASK_AF (0x08)
#define CAMERA_MASK_SHUTTER (0x10)
#define CAMERA_MASK_VIDEO (0x20)
#define CAMERA_MASK_POWERED (0x40)


// maximum number of outstanding commands
// others issued beyond this are dropped
#define MAX_COMMANDS 10

// command characters
#define CAMERA_TOTAL_COMMANDS 9

#define CAMERA_COMMAND_NONE        (0x00)
#define CAMERA_COMMAND_ON          (0x01)
#define CAMERA_COMMAND_OFF         (0x02)
#define CAMERA_COMMAND_AF          (0x03)
#define CAMERA_COMMAND_SHUTTER     (0x04)
#define CAMERA_COMMAND_VIDEO_START (0x05)
#define CAMERA_COMMAND_VIDEO_STOP  (0x06)
#define CAMERA_COMMAND_AF_STROBE   (0x07)
#define CAMERA_COMMAND_NULL_STROBE (0x08)	 



// Input:

// PTAD5 is input for power sensing, pulled high
#define CAMERA_IS_POWERED() (!(CAMERA_INPUT_PORT & CAMERA_MASK_POWERED))


// Output:

// CAMERA_OUTPUT_PORT0 toggles the On/Off button
#define CAMERA_SET_ON() {CAMERA_OUTPUT_PORT |= CAMERA_MASK_ON;}
#define CAMERA_UNSET_ON() {CAMERA_OUTPUT_PORT &= ~CAMERA_MASK_ON;}

// CAMERA_OUTPUT_PORT1 toggles auto focus
#define CAMERA_SET_AF() {CAMERA_OUTPUT_PORT |= CAMERA_MASK_AF;}
#define CAMERA_UNSET_AF() {CAMERA_OUTPUT_PORT &= ~CAMERA_MASK_AF;}

// CAMERA_OUTPUT_PORT2 toggles shutter
#define CAMERA_SET_SHUTTER() {CAMERA_OUTPUT_PORT |= CAMERA_MASK_SHUTTER;}
#define CAMERA_UNSET_SHUTTER() {CAMERA_OUTPUT_PORT &= ~CAMERA_MASK_SHUTTER;}

// CAMERA_OUTPUT_PORT3 toggles video
#define CAMERA_SET_VIDEO() {CAMERA_OUTPUT_PORT |= CAMERA_MASK_VIDEO;}
#define CAMERA_UNSET_VIDEO() {CAMERA_OUTPUT_PORT &= ~CAMERA_MASK_VIDEO;}


void Camera_Init(void);


// this function transitions the camera control pins
void Camera_HandleEvents(void);

// a command to the camera queue
void Camera_Send(unsigned char command);

#endif
