/* sd.h
   Provides an inferace to an SD card over an SPI port.
   
   Author: David Collins
   
   Reference:
   Robert Imhoff, info@flashgenie.net
   http://www.flashgenie.net
   
*/

#ifndef HEADER_SD
#define HEADER_SD


// this is the default read block size in bytes
// the block write size must be 512
#define SD_BLOCK_SIZE 512
#define SD_EMPTY_CHAR 0xFF
#define SD_BUSY_CHAR 0x01
#define SD_DATA_START_TOKEN 0xFE


// bit set in SPISR when transfer is complete
#define SPIF 0x80
// bit set in SPISR when SPI register is ready for new data
#define SPTEF 0x20


// loops until data shifting is complete
#define SPI_WAIT_FOR_DATA() while ( !(SPISR & SPIF) );
#define SPI_WAIT_FOR_SPACE() while ( !(SPISR & SPTEF) );

// chip select lines for SD card
#define SPI_CHIP_SELECT 0x08
#define SD_SELECT() PTM &= ~SPI_CHIP_SELECT;
#define SD_DESELECT() PTM |= SPI_CHIP_SELECT;

// detect if an SD card is inserted
//#define SD_IS_CARD_INSERTED() (!(PTM & 0x02))
// currently not operational
#define SD_IS_CARD_INSERTED() (1)

// set the SPI clock to ~400kHz per spec (375kHz) and 12MHz respectively
//#define SD_SET_SPEED_HIGH() SPIBR = 0b00000000;
// prototype setup requires much slower clock 6MHz
#define SD_SET_SPEED_HIGH() SPIBR = 0b00000001;
#define SD_SET_SPEED_LOW() SPIBR = 0b00000101; 


// Initialize microcontroller registers for use with SD card
void SD_Init(void);

// proceeds though an entire SPI shift operation and returns the value
// sent from the slave
unsigned char SPI_ShiftByte(unsigned char input);

// get two-byte R2 response from SD card if write command fails
unsigned short SD_GetStatus(void);

// send an SD command code to the SD card
unsigned char SD_SendCommand(unsigned char command, unsigned long blocknum);

// Send the initialization sequence to the SD card
// returns 0 if successful
unsigned char SD_Card_Init(void);

// Read in a block from the SD card
// Returns 0 on success
unsigned char SD_Read_Block(unsigned char *buffer, unsigned long blocknum);

// Read in a block from the SD card
// Returns 0 on success
unsigned char SD_Write_Block(unsigned char *buffer, unsigned long blocknum);

#endif